@@ -1695,6 +1695,11 @@ void UpdateCommand::setPosition(int trackIndex, int clipIndex, int position)
     m_undoHelper.recordBeforeState();
 }
 
+void UpdateCommand::setRippleAllTracks(bool ripple)
+{
+    m_rippleAllTracks = ripple;
+}
+
 void UpdateCommand::redo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "position" <<
@@ -1761,7 +1766,8 @@ void DetachAudioCommand::redo()
         // Remove audio filters from the video clip.
         for (int i = 0; i < videoClip.filter_count(); i++) {
             Mlt::Filter *filter = videoClip.filter(i);
-            if (filter && filter->is_valid() && !filter->get_int("_loader")) {
+            if (filter && filter->is_valid() && !filter->get_int("_loader")
+                    && !filter->get_int(kShotcutHiddenProperty)) {
                 QmlMetadata *newMeta = MAIN.filterController()->metadataForService(filter);
                 if (newMeta && newMeta->isAudio()) {
                     videoClip.detach(*filter);
@@ -1777,7 +1783,8 @@ void DetachAudioCommand::redo()
         // Remove video filters from the audio clip.
         for (int i = 0; i < audioClip.filter_count(); i++) {
             Mlt::Filter *filter = audioClip.filter(i);
-            if (filter && filter->is_valid() && !filter->get_int("_loader")) {
+            if (filter && filter->is_valid() && !filter->get_int("_loader")
+                    && !filter->get_int(kShotcutHiddenProperty)) {
                 QmlMetadata *newMeta = MAIN.filterController()->metadataForService(filter);
                 if (newMeta && !newMeta->isAudio()) {
                     audioClip.detach(*filter);
@@ -2008,7 +2015,8 @@ void ApplyFiltersCommand::redo()
     QList<QmlMetadata *> m_applyMeta;
     for (int i = 0; i < filtersProducer.filter_count(); i++) {
         Mlt::Filter *filter = filtersProducer.filter(i);
-        if (filter && filter->is_valid() && !filter->get_int("_loader")) {
+        if (filter && filter->is_valid() && !filter->get_int("_loader")
+                && !filter->get_int(kShotcutHiddenProperty)) {
             m_applyMeta.append(MAIN.filterController()->metadataForService(filter));
         }
         delete filter;
@@ -2020,7 +2028,8 @@ void ApplyFiltersCommand::redo()
             // Remove any filters that would be duplicated by the new filters
             for (int i = 0; i < clipInfo->producer->filter_count(); i++) {
                 Mlt::Filter *filter = clipInfo->producer->filter(i);
-                if (filter && filter->is_valid() && !filter->get_int("_loader")) {
+                if (filter && filter->is_valid() && !filter->get_int("_loader")
+                        && !filter->get_int(kShotcutHiddenProperty)) {
                     QmlMetadata *currentMeta = MAIN.filterController()->metadataForService(filter);
                     for (int j = 0; j < m_applyMeta.size(); j++) {
                         if (m_applyMeta[j] == currentMeta) {
@@ -2034,6 +2043,8 @@ void ApplyFiltersCommand::redo()
             }
             // Apply the new filters
             MLT.pasteFilters(clipInfo->producer, &filtersProducer);
+            // Report that the filters have changed.
+            m_model.filterAddedOrRemoved(clipInfo->producer);
         } else {
             LOG_ERROR() << "Unable to find clip" << clip.trackIndex << clip.clipIndex;
         }
@@ -2049,7 +2060,8 @@ void ApplyFiltersCommand::undo()
             // Remove existing filters
             for (int i = 0; i < clipInfo->producer->filter_count(); i++) {
                 Mlt::Filter *filter = clipInfo->producer->filter(i);
-                if (filter && filter->is_valid() && !filter->get_int("_loader")) {
+                if (filter && filter->is_valid() && !filter->get_int("_loader")
+                        && !filter->get_int(kShotcutHiddenProperty)) {
                     clipInfo->producer->detach(*filter);
                     i--;
                 }
@@ -2059,7 +2071,10 @@ void ApplyFiltersCommand::undo()
             Mlt::Producer previousProducer(MLT.profile(), "xml-string",
                                            m_prevFilters[clip].toUtf8().constData());
             if (previousProducer.is_valid()) {
+                // Apply the previous filters
                 MLT.pasteFilters(clipInfo->producer, &previousProducer);
+                // Report that the filters have changed.
+                m_model.filterAddedOrRemoved(clipInfo->producer);
             } else {
                 LOG_ERROR() << "Unable to restore previous producer";
             }
