@@ -49,7 +49,8 @@
 #include <QClipboard>
 #include <QActionGroup>
 
-static const int kInOutChangedTimeoutMs = 100;
+static const auto kInOutChangedTimeoutMs = 100;
+static const auto kTilePaddingPx = 10;
 
 #define kDetailedMode "detailed"
 #define kIconsMode "icons"
@@ -73,7 +74,7 @@ public:
         const QImage thumb = index.data(Qt::DecorationRole).value<QImage>();
         const QString setting = Settings.playlistThumbnails();
         const int lineHeight = painter->fontMetrics().height();
-        const bool roomEnoughForAllDetails = lineHeight * 5 < thumb.height();
+        const bool roomEnoughForAllDetails = lineHeight * 3 < thumb.height() + kTilePaddingPx;
         const QFont oldFont = painter->font();
         QFont boldFont(oldFont);
         boldFont.setBold(true);
@@ -100,7 +101,7 @@ public:
         painter->setFont(oldFont);
 
         QRect centeredTextRect = option.rect;
-        centeredTextRect.setHeight(lineHeight * (roomEnoughForAllDetails ? 5 : 3));
+        centeredTextRect.setHeight(lineHeight * (roomEnoughForAllDetails ? 3 : 2) + kTilePaddingPx);
         centeredTextRect.moveCenter(option.rect.center());
 
         QRect textRect = centeredTextRect;
@@ -123,10 +124,7 @@ public:
                                                                 PlaylistModel::FIELD_DURATION).toString()));
         if (roomEnoughForAllDetails) {
             textPoint.setY(textPoint.y() + lineHeight);
-            painter->drawText(textPoint, tr("In: %1").arg(index.data(PlaylistModel::FIELD_IN).toString()));
-            textPoint.setY(textPoint.y() + lineHeight);
-            painter->drawText(textPoint, tr("Start: %1").arg(index.data(
-                                                                 PlaylistModel::FIELD_START).toString()));
+            painter->drawText(textPoint, tr("Date: %1").arg(index.data(PlaylistModel::FIELD_DATE).toString()));
         }
     }
 
@@ -137,9 +135,8 @@ public:
         Q_UNUSED(index);
         const bool doubleHeight = Settings.playlistThumbnails() == "tall"
                                   || Settings.playlistThumbnails() == "large";
-        const int spacing = 10;
         return QSize(m_view->viewport()->width(),
-                     PlaylistModel::THUMBNAIL_HEIGHT * (doubleHeight ? 2 : 1) + spacing);
+                     PlaylistModel::THUMBNAIL_HEIGHT * (doubleHeight ? 2 : 1) + kTilePaddingPx);
     }
 
 private slots:
@@ -204,6 +201,13 @@ PlaylistDock::PlaylistDock(QWidget *parent) :
     QMenu *sortByMenu = m_mainMenu->addMenu(tr("Sort"));
     sortByMenu->addAction(Actions["playlistSortByNameAction"]);
     sortByMenu->addAction(Actions["playlistSortByDateAction"]);
+    QMenu *columnsMenu = m_mainMenu->addMenu(tr("Columns"));
+    columnsMenu->addAction(Actions["playlistColumnsToggleThumbnailsAction"]);
+    columnsMenu->addAction(Actions["playlistColumnsToggleClipAction"]);
+    columnsMenu->addAction(Actions["playlistColumnsToggleInAction"]);
+    columnsMenu->addAction(Actions["playlistColumnsToggleDurationAction"]);
+    columnsMenu->addAction(Actions["playlistColumnsToggleStartAction"]);
+    columnsMenu->addAction(Actions["playlistColumnsToggleDateAction"]);
     Actions.loadFromMenu(m_mainMenu);
 
     DockToolBar *toolbar = new DockToolBar(tr("Playlist Controls"));
@@ -727,6 +731,60 @@ void PlaylistDock::setupActions()
         action->setEnabled(m_model.rowCount() > 8);
     });
     Actions.add("playlistSelectClip9Action", action);
+
+    action = new QAction(tr("Thumbnails"), this);
+    action->setChecked(Settings.playlistShowColumn("thumbnails"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, [ = ](bool checked) {
+        Settings.setPlaylistShowColumn("thumbnails", checked);
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_THUMBNAIL, !checked);
+    });
+    Actions.add("playlistColumnsToggleThumbnailsAction", action);
+
+    action = new QAction(tr("Clip"), this);
+    action->setChecked(Settings.playlistShowColumn("clip"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, [ = ](bool checked) {
+        Settings.setPlaylistShowColumn("clip", checked);
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_RESOURCE, !checked);
+    });
+    Actions.add("playlistColumnsToggleClipAction", action);
+
+    action = new QAction(tr("In"), this);
+    action->setChecked(Settings.playlistShowColumn("in"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, [ = ](bool checked) {
+        Settings.setPlaylistShowColumn("in", checked);
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_IN, !checked);
+    });
+    Actions.add("playlistColumnsToggleInAction", action);
+
+    action = new QAction(tr("Duration"), this);
+    action->setChecked(Settings.playlistShowColumn("duration"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, [ = ](bool checked) {
+        Settings.setPlaylistShowColumn("duration", checked);
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_DURATION, !checked);
+    });
+    Actions.add("playlistColumnsToggleDurationAction", action);
+
+    action = new QAction(tr("Start"), this);
+    action->setChecked(Settings.playlistShowColumn("start"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, [ = ](bool checked) {
+        Settings.setPlaylistShowColumn("start", checked);
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_START, !checked);
+    });
+    Actions.add("playlistColumnsToggleStartAction", action);
+
+    action = new QAction(tr("Date"), this);
+    action->setChecked(Settings.playlistShowColumn("date"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, [ = ](bool checked) {
+        Settings.setPlaylistShowColumn("date", checked);
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_DATE, !checked);
+    });
+    Actions.add("playlistColumnsToggleDateAction", action);
 }
 
 int PlaylistDock::position()
@@ -834,6 +892,104 @@ void PlaylistDock::moveClipDown()
     }
 }
 
+void PlaylistDock::addFiles(int row, const QList<QUrl> &urls)
+{
+    auto resetIndex = true;
+    ResourceDialog dialog(this);
+    LongUiTask longTask(tr("Add Files"));
+    int insertNextAt = row;
+    bool first = true;
+    QStringList fileNames = Util::sortedFileList(Util::expandDirectories(urls));
+    qsizetype i = 0, count = fileNames.size();
+    for (auto &path : fileNames) {
+        if (MAIN.isSourceClipMyProject(path)) continue;
+        longTask.reportProgress(Util::baseName(path), i++, count);
+        if (MLT.checkFile(path)) {
+            emit showStatusMessage(tr("Failed to open ").append(path));
+            continue;
+        }
+        Mlt::Producer p;
+        if (path.endsWith(".mlt") || path.endsWith(".xml")) {
+            if (Settings.playerGPU() && MLT.profile().is_explicit()) {
+                Mlt::Profile testProfile;
+                Mlt::Producer producer(testProfile, path.toUtf8().constData());
+                if (testProfile.width() != MLT.profile().width()
+                        || testProfile.height() != MLT.profile().height()
+                        || Util::isFpsDifferent(MLT.profile().fps(), testProfile.fps())) {
+                    emit showStatusMessage(tr("Failed to open ").append(path));
+                    continue;
+                }
+            }
+            p = Mlt::Producer(MLT.profile(), path.toUtf8().constData());
+            if (p.is_valid()) {
+                // Convert MLT XML to a virtual clip.
+                p.set(kShotcutVirtualClip, 1);
+                p.set("resource", path.toUtf8().constData());
+                first = false;
+            }
+        } else {
+            p = Mlt::Producer(MLT.profile(), path.toUtf8().constData());
+        }
+        if (p.is_valid()) {
+            Mlt::Producer *producer = &p;
+            if (first) {
+                first = false;
+                if (!MLT.producer() || !MLT.producer()->is_valid()) {
+                    Mlt::Properties properties;
+                    properties.set(kShotcutSkipConvertProperty, 1);
+                    MAIN.open(path, &properties, false);
+                    if (MLT.producer() && MLT.producer()->is_valid()) {
+                        producer = MLT.producer();
+                        first = true;
+                    }
+                }
+            }
+            producer = MLT.setupNewProducer(producer);
+            producer->set(kShotcutSkipConvertProperty, 1);
+            if (!MLT.isLiveProducer(producer) || producer->get_int(kShotcutVirtualClip)) {
+                ProxyManager::generateIfNotExists(*producer);
+                if (row == -1)
+                    MAIN.undoStack()->push(new Playlist::AppendCommand(m_model, MLT.XML(producer)));
+                else
+                    MAIN.undoStack()->push(new Playlist::InsertCommand(m_model, MLT.XML(producer), insertNextAt++));
+            } else {
+                LongUiTask::cancel();
+                DurationDialog durationDialog(this);
+                durationDialog.setDuration(MLT.profile().fps() * 5);
+                if (durationDialog.exec() == QDialog::Accepted) {
+                    producer->set_in_and_out(0, durationDialog.duration() - 1);
+                    if (row == -1)
+                        MAIN.undoStack()->push(new Playlist::AppendCommand(m_model, MLT.XML(producer)));
+                    else
+                        MAIN.undoStack()->push(new Playlist::InsertCommand(m_model, MLT.XML(producer), insertNextAt++));
+                }
+            }
+            if (first) {
+                first = false;
+                setIndex(0);
+                resetIndex = false;
+            }
+            dialog.add(producer);
+            delete producer;
+        }
+    }
+    if (Settings.showConvertClipDialog() && dialog.producerCount() > 1 && dialog.hasTroubleClips()) {
+        dialog.selectTroubleClips();
+        dialog.setWindowTitle(tr("Dropped Files"));
+        longTask.cancel();
+        dialog.exec();
+    } else if (Settings.showConvertClipDialog() && dialog.producerCount() == 1) {
+        Mlt::Producer producer = dialog.producer(0);
+        QString convertAdvice = Util::getConversionAdvice(&producer);
+        if (!convertAdvice.isEmpty()) {
+            longTask.cancel();
+            Util::offerSingleFileConversion(convertAdvice, &producer, this);
+        }
+    }
+    if (resetIndex)
+        resetPlaylistIndex();
+}
+
 void PlaylistDock::onInsertCutActionTriggered()
 {
     if (MLT.isClip() || MLT.savedProducer()) {
@@ -1100,99 +1256,12 @@ void PlaylistDock::onPlaylistCleared()
 
 void PlaylistDock::onDropped(const QMimeData *data, int row)
 {
-    bool resetIndex = true;
     if (data && data->hasUrls()) {
-        ResourceDialog dialog(this);
-        LongUiTask longTask(tr("Add Files"));
-        int insertNextAt = row;
-        bool first = true;
-        QStringList fileNames = Util::sortedFileList(Util::expandDirectories(data->urls()));
-        qsizetype i = 0, count = fileNames.size();
-        for (auto &path : fileNames) {
-            if (MAIN.isSourceClipMyProject(path)) continue;
-            longTask.reportProgress(Util::baseName(path), i++, count);
-            if (MLT.checkFile(path)) {
-                emit showStatusMessage(tr("Failed to open ").append(path));
-                continue;
-            }
-            Mlt::Producer p;
-            if (path.endsWith(".mlt") || path.endsWith(".xml")) {
-                if (Settings.playerGPU() && MLT.profile().is_explicit()) {
-                    Mlt::Profile testProfile;
-                    Mlt::Producer producer(testProfile, path.toUtf8().constData());
-                    if (testProfile.width() != MLT.profile().width()
-                            || testProfile.height() != MLT.profile().height()
-                            || Util::isFpsDifferent(MLT.profile().fps(), testProfile.fps())) {
-                        emit showStatusMessage(tr("Failed to open ").append(path));
-                        continue;
-                    }
-                }
-                p = Mlt::Producer(MLT.profile(), path.toUtf8().constData());
-                if (p.is_valid()) {
-                    // Convert MLT XML to a virtual clip.
-                    p.set(kShotcutVirtualClip, 1);
-                    p.set("resource", path.toUtf8().constData());
-                    first = false;
-                }
-            } else {
-                p = Mlt::Producer(MLT.profile(), path.toUtf8().constData());
-            }
-            if (p.is_valid()) {
-                Mlt::Producer *producer = &p;
-                if (first) {
-                    first = false;
-                    if (!MLT.producer() || !MLT.producer()->is_valid()) {
-                        Mlt::Properties properties;
-                        properties.set(kShotcutSkipConvertProperty, 1);
-                        MAIN.open(path, &properties, false);
-                        if (MLT.producer() && MLT.producer()->is_valid()) {
-                            producer = MLT.producer();
-                            first = true;
-                        }
-                    }
-                }
-                producer = MLT.setupNewProducer(producer);
-                producer->set(kShotcutSkipConvertProperty, 1);
-                if (!MLT.isLiveProducer(producer) || producer->get_int(kShotcutVirtualClip)) {
-                    ProxyManager::generateIfNotExists(*producer);
-                    if (row == -1)
-                        MAIN.undoStack()->push(new Playlist::AppendCommand(m_model, MLT.XML(producer)));
-                    else
-                        MAIN.undoStack()->push(new Playlist::InsertCommand(m_model, MLT.XML(producer), insertNextAt++));
-                } else {
-                    LongUiTask::cancel();
-                    DurationDialog durationDialog(this);
-                    durationDialog.setDuration(MLT.profile().fps() * 5);
-                    if (durationDialog.exec() == QDialog::Accepted) {
-                        producer->set_in_and_out(0, durationDialog.duration() - 1);
-                        if (row == -1)
-                            MAIN.undoStack()->push(new Playlist::AppendCommand(m_model, MLT.XML(producer)));
-                        else
-                            MAIN.undoStack()->push(new Playlist::InsertCommand(m_model, MLT.XML(producer), insertNextAt++));
-                    }
-                }
-                if (first) {
-                    first = false;
-                    setIndex(0);
-                    resetIndex = false;
-                }
-                dialog.add(producer);
-                delete producer;
-            }
-        }
-        if (Settings.showConvertClipDialog() && dialog.producerCount() > 1 && dialog.hasTroubleClips()) {
-            dialog.selectTroubleClips();
-            dialog.setWindowTitle(tr("Dropped Files"));
-            longTask.cancel();
-            dialog.exec();
-        } else if (Settings.showConvertClipDialog() && dialog.producerCount() == 1) {
-            Mlt::Producer producer = dialog.producer(0);
-            QString convertAdvice = Util::getConversionAdvice(&producer);
-            if (!convertAdvice.isEmpty()) {
-                longTask.cancel();
-                Util::offerSingleFileConversion(convertAdvice, &producer, this);
-            }
-        }
+        // Use QTimer to workaround stupid drag from Windows Explorer bug
+        const auto &urls = data->urls();
+        QTimer::singleShot(0, this, [ = ]() {
+            addFiles(row, urls);
+        });
     } else if (data && data->hasFormat(Mlt::XmlMimeType)) {
         if (MLT.producer() && MLT.producer()->is_valid()) {
             if (MLT.producer()->type() == mlt_service_playlist_type) {
@@ -1226,8 +1295,6 @@ void PlaylistDock::onDropped(const QMimeData *data, int row)
             }
         }
     }
-    if (resetIndex)
-        resetPlaylistIndex();
 }
 
 void PlaylistDock::onMoveClip(int from, int to)
@@ -1320,6 +1387,17 @@ void PlaylistDock::updateViewMode()
         m_model.setViewMode(PlaylistModel::Detailed);
         m_view = ui->tableView;
         ui->tableView->setModel(&m_model);
+
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_THUMBNAIL,
+                                       !Settings.playlistShowColumn("thumbnails"));
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_RESOURCE,
+                                       !Settings.playlistShowColumn("clip"));
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_IN, !Settings.playlistShowColumn("in"));
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_DURATION,
+                                       !Settings.playlistShowColumn("duration"));
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_START, !Settings.playlistShowColumn("start"));
+        ui->tableView->setColumnHidden(PlaylistModel::COLUMN_DATE, !Settings.playlistShowColumn("date"));
+
         ui->tableView->resizeColumnsToContents();
         ui->tableView->show();
 
