@@ -36,6 +36,7 @@
 static const char *kProxySubfolder = "proxies";
 static const char *kProxyVideoExtension = ".mp4";
 static const char *kGoProProxyVideoExtension = ".LRV";
+static const char *kDJIProxyVideoExtension = ".LRF";
 static const char *kProxyPendingVideoExtension = ".pending.mp4";
 static const char *kProxyImageExtension = ".jpg";
 static const char *kProxyPendingImageExtension = ".pending.jpg";
@@ -98,6 +99,7 @@ void ProxyManager::generateVideoProxy(Mlt::Producer &producer, bool fullRange, S
     file.close();
 
     args << "-loglevel" << "verbose";
+    args << "-noautorotate";
     args << "-i" << resource;
     args << "-max_muxing_queue_size" << "9999";
     // transcode all streams except data, subtitles, and attachments
@@ -411,6 +413,9 @@ bool ProxyManager::fileExists(Mlt::Producer &producer)
         if (QFile::exists(GoProProxyFilePath(producer.get("resource")))) {
             return true;
         }
+        if (QFile::exists(DJIProxyFilePath(producer.get("resource")))) {
+            return true;
+        }
         fileName = Util::getHash(producer) + kProxyVideoExtension;
     } else if (isValidImage(producer)) {
         fileName = Util::getHash(producer) + kProxyImageExtension;
@@ -486,17 +491,25 @@ bool ProxyManager::generateIfNotExists(Mlt::Producer &producer, bool replace)
         if (ProxyManager::fileExists(producer)) {
             QString service = QString::fromLatin1(producer.get("mlt_service"));
             QDir projectDir(MLT.projectFolder());
-            QString fileName;
+            QString fileName = Util::getHash(producer) + kProxyVideoExtension;
+            QDir proxyDir(Settings.proxyFolder());
             if (service.startsWith("avformat")) {
                 auto gopro = GoProProxyFilePath(producer.get("resource"));
-                if (QFile::exists(gopro)) {
-                    producer.set(kIsProxyProperty, 1);
-                    producer.set(kMetaProxyProperty, 1);
-                    producer.set(kOriginalResourceProperty, producer.get("resource"));
-                    producer.set("resource", gopro.toUtf8().constData());
-                    return true;
-                } else {
-                    fileName = Util::getHash(producer) + kProxyVideoExtension;
+                auto dji = DJIProxyFilePath(producer.get("resource"));
+                if (!projectDir.exists(fileName) && !proxyDir.exists(fileName)) {
+                    if (QFile::exists(gopro)) {
+                        producer.set(kIsProxyProperty, 1);
+                        producer.set(kMetaProxyProperty, 1);
+                        producer.set(kOriginalResourceProperty, producer.get("resource"));
+                        producer.set("resource", gopro.toUtf8().constData());
+                        return true;
+                    } else if (QFile::exists(dji)) {
+                        producer.set(kIsProxyProperty, 1);
+                        producer.set(kMetaProxyProperty, 1);
+                        producer.set(kOriginalResourceProperty, producer.get("resource"));
+                        producer.set("resource", dji.toUtf8().constData());
+                        return true;
+                    }
                 }
             } else if (isValidImage(producer)) {
                 fileName = Util::getHash(producer) + kProxyImageExtension;
@@ -510,7 +523,6 @@ bool ProxyManager::generateIfNotExists(Mlt::Producer &producer, bool replace)
                 ::utime(projectDir.filePath(fileName).toUtf8().constData(), nullptr);
                 producer.set("resource", projectDir.filePath(fileName).toUtf8().constData());
             } else {
-                QDir proxyDir(Settings.proxyFolder());
                 ::utime(proxyDir.filePath(fileName).toUtf8().constData(), nullptr);
                 producer.set("resource", proxyDir.filePath(fileName).toUtf8().constData());
             }
@@ -698,3 +710,12 @@ QString ProxyManager::GoProProxyFilePath(const QString &resource)
     LOG_DEBUG() << result;
     return result;
 }
+
+QString ProxyManager::DJIProxyFilePath(const QString &resource)
+{
+    auto fi = QFileInfo(resource);
+    auto base = fi.baseName();
+    auto result = fi.absoluteDir().filePath(base + kDJIProxyVideoExtension);
+    LOG_DEBUG() << result;
+    return result;
+}
