@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2023 Meltytech, LLC
+ * Copyright (c) 2012-2024 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "mltcontroller.h"
 #include "shotcut_mlt_properties.h"
 #include "dialogs/filedatedialog.h"
+#include "dialogs/listselectiondialog.h"
 #include "jobqueue.h"
 #include "jobs/ffprobejob.h"
 #include "jobs/ffmpegjob.h"
@@ -38,6 +39,7 @@
 #include "jobs/bitrateviewerjob.h"
 
 #include <QtWidgets>
+#include <limits>
 
 static const auto kHandleSeconds = 15.0;
 
@@ -100,7 +102,7 @@ void AvformatProducerWidget::updateDuration()
         auto duration = m_producer->get_int(kFilterOutProperty) - m_producer->get_int(
                             kFilterInProperty) + 1;
         ui->timelineDurationLabel->show();
-        ui->timelineDurationText->setText(m_producer->frames_to_time(duration));
+        ui->timelineDurationText->setText(m_producer->frames_to_time(duration, Settings.timeFormat()));
         ui->timelineDurationText->show();
     } else {
         ui->timelineDurationLabel->hide();
@@ -136,6 +138,9 @@ void AvformatProducerWidget::offerConvert(QString message, bool set709Convert, b
                 m_producer->get_length() - 1;
     dialog.setSubClipChecked(setSubClip && (m_producer->get_in() > 0
                                             || m_producer->get_out() < m_producer->get_length() - 1));
+    auto fps = Util::getAndroidFrameRate(m_producer.get());
+    if (fps > 0.0)
+        dialog.setFrameRate(fps);
     Transcoder transcoder;
     transcoder.addProducer(m_producer.data());
     transcoder.convert(dialog);
@@ -258,9 +263,16 @@ void AvformatProducerWidget::reloadProducerValues()
     ui->tabWidget->setTabEnabled(0, false);
     ui->tabWidget->setTabEnabled(1, false);
     ui->tabWidget->setTabEnabled(2, false);
-    if (m_defaultDuration == -1)
+    if (m_defaultDuration == -1) {
         m_defaultDuration = m_producer->get_length();
 
+        // Special hack for images
+        if (m_defaultDuration == std::numeric_limits<int>().max() && fps() == 1.0) {
+            m_defaultDuration = qRound(MLT.profile().fps() * Settings.imageDuration());
+            m_producer->set("length", m_defaultDuration);
+        }
+    }
+
     double warpSpeed = Util::GetSpeedFromProducer(producer());
     QString resource = Util::GetFilenameFromProducer(producer());
     QString caption = Util::updateCaption(m_producer.data());
@@ -268,7 +280,7 @@ void AvformatProducerWidget::reloadProducerValues()
     ui->filenameLabel->setCursorPosition(caption.length());
     ui->filenameLabel->setToolTip(resource);
     ui->notesTextEdit->setPlainText(QString::fromUtf8(m_producer->get(kCommentProperty)));
-    ui->durationSpinBox->setValue(m_producer->get_length());
+    ui->durationSpinBox->setValue(m_defaultDuration);
     updateDuration();
     m_recalcDuration = false;
     ui->speedSpinBox->setValue(warpSpeed);
@@ -668,6 +680,7 @@ void AvformatProducerWidget::on_menuButton_clicked()
     menu.addAction(ui->actionFFmpegIntegrityCheck);
     menu.addAction(ui->actionFFmpegConvert);
     menu.addAction(ui->actionExtractSubclip);
+    menu.addAction(ui->actionExtractSubtitles);
     menu.addAction(ui->actionSetFileDate);
     if (Util::GetFilenameFromProducer(producer()).toLower().endsWith(".mp4")
             || Util::GetFilenameFromProducer(producer()).toLower().endsWith(".mov")) {
@@ -731,6 +744,9 @@ void AvformatProducerWidget::on_actionFFmpegConvert_triggered()
     dialog.setWindowModality(QmlApplication::dialogModality());
     dialog.set709Convert(ui->videoTableWidget->item(5, 1)->data(Qt::UserRole).toInt() > 7);
     dialog.showSubClipCheckBox();
+    auto fps = Util::getAndroidFrameRate(m_producer.get());
+    if (fps > 0.0)
+        dialog.setFrameRate(fps);
     Transcoder transcoder;
     transcoder.addProducer(m_producer.data());
     transcoder.convert(dialog);
@@ -1046,6 +1062,90 @@ void AvformatProducerWidget::on_actionExtractSubclip_triggered()
     }
 }
 
+void AvformatProducerWidget::on_actionExtractSubtitles_triggered()
+{
+    QStringList subtitles;
+    int nb_streams = m_producer->get_int("meta.media.nb_streams");
+    int subtitleCount = 0;
+    for (int i = 0; i < nb_streams; i++) {
+        QString key = QString("meta.media.%1.codec.name").arg(i);
+        QString codec(m_producer->get(key.toLatin1().constData()));
+        if (codec == "srt") {
+            subtitleCount++;
+            QString langKey = QString("meta.attr.%1.stream.language.markup").arg(i);
+            QString lang(m_producer->get(langKey.toLatin1().constData()));
+            if (lang.isEmpty()) {
+                subtitles << tr("Track %1").arg(subtitleCount);
+            } else {
+                subtitles << tr("Track %1 (%2)").arg(subtitleCount).arg(lang);
+            }
+        }
+    }
+    QString caption = tr("Export Subtitles...");
+    if (subtitles.size() <= 0) {
+        QMessageBox::warning(this, caption, tr("No subtitles found"));
+        return;
+    }
+
+    // Prompt the user to select the subtitle stream
+    ListSelectionDialog dialog(subtitles, this);
+    dialog.setWindowModality(QmlApplication::dialogModality());
+    dialog.setWindowTitle(caption);
+    dialog.setSelection(QStringList(subtitles[0]));
+    if (dialog.exec() != QDialog::Accepted || dialog.selection().isEmpty()) {
+        LOG_WARNING() << "No subtitles selected";
+        return;
+    }
+    QStringList subSelection = dialog.selection();
+
+    // Prompt the user for the directory to save the file(s)
+    QString resource = Util::GetFilenameFromProducer(producer());
+    QFileInfo fi(resource);
+    QString pathTemplate = QFileDialog::getExistingDirectory(this, caption, Settings.savePath(),
+                                                             Util::getFileDialogOptions());
+    if (pathTemplate.isEmpty()) {
+        LOG_WARNING() << "No path specified";
+        return;
+    }
+    // Extract the subtitle(s)
+    pathTemplate = QDir::toNativeSeparators(pathTemplate);
+    pathTemplate.append("/%1");
+    pathTemplate = pathTemplate.arg(fi.completeBaseName());
+    pathTemplate.append(" - %1.srt");
+    for (int s = 0; s < subSelection.size(); s++) {
+        int subtitleCount = 0;
+        for (int i = 0; i < nb_streams; i++) {
+            QString key = QString("meta.media.%1.codec.name").arg(i);
+            QString codec(m_producer->get(key.toLatin1().constData()));
+            if (codec == "srt") {
+                subtitleCount++;
+                QString subText;
+                QString langKey = QString("meta.attr.%1.stream.language.markup").arg(i);
+                QString lang(m_producer->get(langKey.toLatin1().constData()));
+                if (lang.isEmpty()) {
+                    subText = tr("Track %1").arg(subtitleCount);
+                } else {
+                    subText = tr("Track %1 (%2)").arg(subtitleCount).arg(lang);
+                }
+                if (subText == subSelection[s]) {
+                    QString path = pathTemplate.arg(subText);
+                    if (Util::warnIfNotWritable(path, this, caption))
+                        return;
+                    // Make an FFMpeg job
+                    QStringList ffmpegArgs;
+                    QString streamSelect = QString("0:s:%1").arg(subtitleCount - 1);
+                    ffmpegArgs << "-loglevel" << "verbose";
+                    ffmpegArgs << "-i" << resource;
+                    ffmpegArgs << "-map" << streamSelect;
+                    ffmpegArgs << "-y" << path;
+                    FfmpegJob *ffmpegJob = new FfmpegJob(path, ffmpegArgs, false);
+                    ffmpegJob->setLabel(tr("Extract subtitles %1").arg(Util::baseName(path)));
+                    JOBS.add(ffmpegJob);
+                }
+            }
+        }
+    }
+}
 
 void AvformatProducerWidget::on_actionSetFileDate_triggered()
 {
@@ -1117,6 +1217,9 @@ void AvformatProducerWidget::on_actionMakeProxy_triggered()
         scan = ui->fieldOrderComboBox->currentIndex() ? ProxyManager::InterlacedTopFieldFirst
                : ProxyManager::InterlacedBottomFieldFirst;
 
+    // If rotation is 90 or 270, swap aspect ratio since auto-rotate is turned off
+    if (ui->rotationComboBox->currentIndex() % 2 == 1)
+        aspectRatio = aspectRatio.transposed();
     ProxyManager::generateVideoProxy(*producer(), fullRange, scan, aspectRatio);
 }
 
